import os
import pandas as pd
import rule_engine
import report

def read_rule_file(metadata_filename):
    config_file = os.path.join(report.__path__[0], "Elasticsearch_Review_Rules.xlsx")
    #file = "report/Elasticsearch_Review_Rules.xlsx"
    data = pd.read_excel(config_file)
    data = data[data["Metadata-Filename"] == metadata_filename]
    return data


def create_rule(rules_text):
    rules = {}
    context = rule_engine.Context(resolver=rule_engine.resolve_attribute)

    for r in rules_text.index:
        rule_text = str(rules_text["Rule"][r])
        #print(rules_text["Name"][r] + ":" + rule_text)
        rule = rule_engine.Rule(rule_text, context=context)
        rules[rules_text["Name"][r]] = rule

    return rules


def apply_rule(config, rules_text):
    recommendation = {}
    rules = create_rule(rules_text)
    for r in rules:
        m = rules[r].matches(config)
        if m:
            recommendation[r] = str(rules_text.loc[rules_text["Name"] == r]["True-Recommendation"].values[0])
        else:
            recommendation[r] = str(rules_text.loc[rules_text["Name"] == r]["False-Recommendation"].values[0])
        #print("Rec: " + recommendation[r])
    return recommendation


def get_recommendation(metadata_filename, domain_config):
    rules_text = read_rule_file(metadata_filename)
    rec = apply_rule(domain_config, rules_text)
    return rec

def create_rules(rules_text):
    rules = {}
    #print(rules_text)
    for r in rules_text.index:
        rule_text = str(rules_text["Rule"][r])
        #print(rules_text["Name"][r] + ":" + rule_text)
        if rule_text != 'nan':
            rule = rule_engine.Rule(rule_text)
            rules[rules_text["Name"][r]] = rule

    return rules

def create_data_dic(rules_text,data):
    data_dic = {}
    #print("data is:" + str(data))
    for r in rules_text.index:
        metadata_field = str(rules_text["Metadata-Field"][r])
        entity_field = str(rules_text["EntityField"][r])
        rule_text = str(rules_text["Rule"][r])
        show_value = str(rules_text["Show-Value"][r])
        fields = metadata_field.split("#")
        current = {}
        i = 0
        #print(" fields is:" + str(fields))
        for f in fields:
            #print("field is: " + f)
            if i == 0:
                if f in data:
                    current = data[f]
                else:
                    break
            else:
                if f in current:
                    current = current[f]
                else:
                    break;
            i += 1

        #print("final value of : " + metadata_field + " is: " +str(current))
        data_dic[entity_field] = current

    return data_dic

def apply_rule_for_data_from_json(rules_text,data):
    recommendation = {}
    rules = create_rules(rules_text)
    print("rules : " + str(rules))
    data_dic = create_data_dic(rules_text, data)
    print("data : " + str(data_dic))
    for r in rules_text["Name"]:
        rec_entry = {}
        rec_entry["recommendation"] = "No Recommendation"
        metadata_field = rules_text.loc[rules_text["Name"] == r]["Metadata-Field"].values[0]
        rec_entry["field"] = metadata_field
        if(r in rules):
            m = rules[r].matches(data_dic)
            if m:
                rec_entry["recommendation"] = str(rules_text.loc[rules_text["Name"] == r]["True-Recommendation"].values[0])
            else:
                rec_entry["recommendation"] = str(rules_text.loc[rules_text["Name"] == r]["False-Recommendation"].values[0])
        if(rules_text.loc[rules_text["Name"] == r]["Show-Value"].values[0]):
            field = rules_text.loc[rules_text["Name"] == r]["EntityField"].values[0]
            rec_entry["original"] = data_dic[field]
        else:
            rec_entry["original"] = ""
        recommendation[r] = rec_entry
    return recommendation

def apply_rule_for_data_from_csv(rules_text,df):
    recommendation = {}
    for r in rules_text["Name"]:
        rec_entry = {"recommendation": "No Recommendation"}
        rec_entry["field"] = rules_text.loc[rules_text["Name"] == r]["EntityField"].values[0]
        rec_entry["original"] = ""
        rule = rules_text.loc[rules_text["Name"] == r]["Rule"].values[0]
        print("csv rule: " + rule)
        matched = eval(rule)
        if len(matched.index) > 0:
            rec_entry["recommendation"] = str(rules_text.loc[rules_text["Name"] == r]["True-Recommendation"].values[0])
        else:
            rec_entry["recommendation"] = str(rules_text.loc[rules_text["Name"] == r]["False-Recommendation"].values[0])
        recommendation[r] = rec_entry

    return recommendation

def get_recommendation_from_data(metadata_filename,data):
    rules_text = read_rule_file(metadata_filename)
    rec = None
    if metadata_filename.endswith("json"):
        rec = apply_rule_for_data_from_json(rules_text,data)
    elif metadata_filename.endswith("csv"):
        rec = apply_rule_for_data_from_csv(rules_text,data)
    else:
        print("ERROR::: Invalid metadata data file")
    return rec
